<?php

require_once ("Files.interface.inc");

class Local_files implements Files{

    private $oProperties;

    function __construct($properties){
        $this->oProperties = $properties;
    }

    /**
     *This method return the extension of a file.
     *@param string $sFilePath File's path.
     *@return string|boolean hash sha1 of file content or false if the file doesn't exist.
     */
    public function getFileEtag($sFilePath){
        if(file_exists($sFilePath)){
            return sha1(file_get_contents($sFilePath), false);
        } else {
            return false;
        }

    }

    /**
     *verify if a file exists.
     *@param string $sFilePath File's path.
     *@return boolean true if file exists, false else.
     */
    public function file_exists ($sFilePath){
        return file_exists($sFilePath);
    }
    /**
     *Write a string in a file.
     *@param string $sFilePath File's path.
     *@param string $sData string to write in this file.
     *@param integer $iFlags flags to write in file (http://php.net/manual/fr/function.file-put-contents.php) Default : 0.
     *@param resource $mContext result de stream_context_create().
     *@return integer|boolean number of byte written, or false if the process encounter an error.
     */
    public function file_put_contents ($sFilePath, $sData, $iFlags = 0, $mContext = null){
        return file_put_contents($sFilePath, $sData, $iFlags, $mContext);
    }
    /**
     *read a file and return content in a string.
     *@param string $sFilePath File's path.
     *@param boolean $bUseIncludePath use include var to resolve a relative path.
     *@param resource $mContext result de stream_context_create().
     *@param integer $iOffset offset in bytes to begin file reading, Default : 0.
     *@param integer $iMaxLen Number of byte to read from the file, Default : -1 = read until the file's end.
     *@return string|boolean file content, or false if the process encounter an error.
     */
    public function file_get_contents ($sFilePath, $bUseIncludePath = FALSE, $mContext = null, $iOffset = 0, $iMaxLen = -1){
        if ($iMaxLen == -1){
            return file_get_contents ($sFilePath, $bUseIncludePath, null, $iOffset);
        } else {
            return file_get_contents ($sFilePath, $bUseIncludePath, null, $iOffset, $iMaxLen);
        }
    }
    /**
     *read a file and return content in a string if eTag matches.
     *@param string $sFilePath File's path.
     *@param string $sEtag Hash Sha1 of file's content.
     *@param boolean $bUseIncludePath use include var to resolve a relative path.
     *@param resource $mContext result de stream_context_create().
     *@param integer $iOffset offset in bytes to begin file reading, Default : 0.
     *@param integer $iMaxLen Number of byte to read from the file, Default : -1 = read until the file's end.
     *@return string|boolean file content, or false if the process encounter an error.
     */
    public function file_get_contents_with_etag ($sFilePath, $sEtag ,$bUseIncludePath = FALSE, $mContext = null, $iOffset = 0, $iMaxLen = -1){
        $sProcessedEtag = $this->getFileEtag($sFilePath);
        if ($sEtag == $sProcessedEtag){
            return $this->file_get_contents($sFilePath, $bUseIncludePath, $mContext, $iOffset, $iMaxLen);
        } else {
            return false;
        }
    }
    /**
     *copy a file.
     *@param string $sSourceFilePath File's path to copy.
     *@param string $sDestFilePath File's destination.
     *@return boolean true on sucess or false if the process encounter an error.
     */
    public function copy ($sSourceFilePath, $sDestFilePath){
        return copy ($sSourceFilePath, $sDestFilePath);
    }
    /**
     *return the size of a file in bytes.
     *@param string $sFilePath File's path.
     *@return integer|boolean File's size in bytes or false if the process encounter an error.
     */
    public function filesize ($sFilePath){
        return filesize($sFilePath);
    }
    /**
     *send file's content into stdout.
     *@param string $sFilePath File's path.
     *@param boolean $bUseIncludePath use include var to resolve a relative path.
     *@param resource $mContext result de stream_context_create().
     *@return integer|boolean File's size in bytes or false if the process encounter an error.
     */
    public function readfile ($sFilePath, $bUseIncludePath = FALSE, $mContext = null){
        return readfile($sFilePath, $bUseIncludePath, $mContext);
    }
    /**
     *move a file.
     *@param string $sSourceFilePath File's path to copy.
     *@param string $sDestFilePath File's destination.
     *@param resource $mContext result de stream_context_create().
     *@return boolean true on sucess or false if the process encounter an error.
     */
    public function rename ($sSourceFilePath, $sDestFilePath, $mContext = null){
        if(empty($mContext)){
            return rename($sSourceFilePath, $sDestFilePath);
        }else{
            return rename($sSourceFilePath, $sDestFilePath, $mContext);
        }
    }
    /**
     *remove a file.
     *@param string $sFilePath File's path.
     *@param resource $mContext result de stream_context_create().
     *@return boolean true on sucess or false if the process encounter an error.
     */
    public function unlink ($sFilePath, $mContext = null){
        if(empty($mContext)){
            return unlink($sFilePath);
        } else {
            return unlink($sFilePath, $mContext);
        }
    }
    /**
     *return the number of second UNIX of the last modification on a file.
     *@param string $sFilePath File's path.
     *@return integer|boolean timestamp on sucess or false if the process encounter an error.
     */
    public function filemtime ($sFilePath){
        return filemtime($sFilePath);
    }
    /**
     *return the fomrated date of the last modification on a file.
     *@param string $sFilePath File's path.
     *@param string $sFormat Format to convert the timestamp.
     *@return string|boolean date string on sucess or false if the process encounter an error.
     */
    public function filemtime_formated($sFilePath, $sFormat){
        return date ($sFormat, $this->filemtime($sFilePath));
    }
    /**
     *return the metadata structure of a file.
     *@param string $sFilePath File's path.
     *@return array filename, size, lastModification, href.
     */
    public function getFileInfos ($sFilePath){
        $iFileSize = $this->filesize($sFilePath);

        $sFileSize = $iFileSize . " octets";
        if($iFileSize > 1024){
            if($iFileSize > 1024*1024){
                if($iFileSize > 1024*1024*1024){
                    $sFileSize = "-";
                } else {
                    $sFileSize = (ceil($iFileSize/(1024*1024))) . "Mo";
                }
            }else{
                $sFileSize = (ceil($iFileSize/1024)) . "Ko";
            }
        }

        $aFileName = explode("/", $sFilePath);

        return array("filename"=>$aFileName[count($aFileName) - 1], "size"=>$sFileSize, "lastModification"=> $this->filemtime_formated ($sFilePath, "d/m/Y H:i:s"), "href" => $this->getProxyPassUrl($sFilePath));
    }
    /**
     *return an url to download the file through Vitis API .
     *@param string $sFilePath File's path.
     *@return string url.
     */
    public function getProxyPassUrl ($sFilePath){
        $date = new DateTime();
        $sDataUrl = $this->oProperties['web_server_name'] . "/rest/vitis/file_downloader?key=[KEY]&eTag=[ETAG]&d=" . $date->getTimestamp();

        $sFileUrl = str_replace("[KEY]", str_replace($this->oProperties['ws_data_dir'], "ws_data" , $sFilePath), $sDataUrl);
        $sFileUrl = str_replace("[ETAG]", sha1(file_get_contents($sFilePath), false), $sFileUrl);

        return $sFileUrl;
    }
    public function scandir($sDirPath, $iSortOrder = SCANDIR_SORT_ASCENDING, $mContext = null){
        if(empty($mContext)){
            return scandir($sDirPath, $iSortOrder);
        }else{
            return scandir($sDirPath, $iSortOrder, $mContext);
        }
    }
    public function mkdir($sDirPath, $iMode = 0777, $bRecursive = FALSE, $mContext = null){
        if(empty($mContext)){
            return mkdir($sDirPath, $iMode, $bRecursive);
        }else{
            return mkdir($sDirPath, $iMode, $bRecursive, $mContext);
        }
    }
    public function rmdir($sDirPath, $mContext = null){
        if(empty($mContext)){
            return rmdir($sDirPath);
        }else{
            return rmdir($sDirPath, $mContext);
        }
    }
    public function is_dir($sDirPath){
        return is_dir($sDirPath);
    }
    public function getFolderInfos($sDirPath){
         if($this->is_dir($sDirPath)){
             $aContentDir = $this->scandir($sDirPath);
             $aTree = $this->getFileInfos($sDirPath);
             $aDir = array();
             for ($i =0 ; $i < count($aContentDir); $i++){
                 if($aContentDir[$i] !== "." && $aContentDir[$i] !== ".."){
                     array_push($aDir, $this->getFolderInfos($sDirPath . "/" . $aContentDir[$i]));
                 }
             }

             $aTree["content"] = $aDir;
             unset($aTree["href"]);
             return $aTree;
         } else {
             return $this->getFileInfos($sDirPath);
         }
    }
    public function clearDir($sDirPath){
        $ouverture = @opendir($sDossier);
        if (!$ouverture)
            return;
        while ($sFichier = readdir($ouverture)) {

            if ($sFichier == '.' || $sFichier == '..')
                continue;
            if ($this->is_dir($sDossier . "/" . $sFichier)) {
                $bCleared = $this->clearDir($sDossier . "/" . $sFichier);
                if (!$bCleared)
                    return false;
            }
            else {
                $bCleared = @unlink($sDossier . "/" . $sFichier);
                if (!$bCleared)
                    return false;
            }
        }
        closedir($ouverture);
        $bCleared = @rmdir($sDossier);
        if (!$bCleared)
            return false;
        return true;
    }
    public function copyDirectory($sDirPathSrc, $sDirPathDest){
        $dir = opendir($src);
        @mkdir($dst);
        while(false !== ( $file = readdir($dir)) ) {
            if (( $file != '.' ) && ( $file != '..' )) {
                if ( is_dir($src . '/' . $file) ) {
                    $this->copyDirectory($src . '/' . $file,$dst . '/' . $file);
                }
                else {
                    copy($src . '/' . $file,$dst . '/' . $file);
                }
            }
        }
        closedir($dir);
    }
}


?>
